﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EnricoClient1.Enrico;

namespace EnricoClient1
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                enricoPortTypeClient client = new enricoPortTypeClient();
                client.Open();
                int day = DateTime.Now.Day;
                int month = DateTime.Now.Month;
                int year = DateTime.Now.Year;

                Console.WriteLine("Public holidays in Germany (Baden-Württemberg) for " + ConvertMonth(month) + " " + year);
                PublicHolidayCollectionType retVal = client.getPublicHolidaysForMonth(month.ToString(), year.ToString(), "ger", "Baden-Württemberg");
                if (!String.IsNullOrEmpty(retVal.error))
                {
                    PrintError(retVal.error);
                }
                else
                {
                    PrintPublicHolidays(retVal.publicHolidays);
                }

                Console.WriteLine("\r\nPublic holidays in Estonia for " + year);
                retVal = client.getPublicHolidaysForYear(year.ToString(), "est", "");
                if (!String.IsNullOrEmpty(retVal.error))
                {
                    PrintError(retVal.error);
                }
                else
                {
                    PrintPublicHolidays(retVal.publicHolidays);
                }

                Console.WriteLine("\r\nPublic holidays in United States of America (District Of Columbia) from 04 July " + year + " (including) to 04 July " + (year + 2) + " (including)");
                DateType from = new DateType();
                from.day = 4;
                from.month = 7;
                from.year = year;
                DateType to = new DateType();
                to.day = 4;
                to.month = 7;
                to.year = year + 2;
                retVal = client.getPublicHolidaysForDateRange(from, to, "usa", "District Of Columbia");
                if (!String.IsNullOrEmpty(retVal.error))
                {
                    PrintError(retVal.error);
                }
                else
                {
                    PrintPublicHolidays(retVal.publicHolidays);
                }

                Console.WriteLine("\r\nIs " + day + " " + ConvertMonth(month) + " " + year + " public holiday in Slovakia? " );
                DateType today = new DateType();
                today.day = day;
                today.month = month;
                today.year = year;
                IsPublicHolidayType ispublicHolidayType = client.isPublicHoliday(today, "svk", "this is ignored by enrico when country has no regions");
                if (!String.IsNullOrEmpty(ispublicHolidayType.error))
                {
                    PrintError(ispublicHolidayType.error);
                }
                else
                {
                    if (ispublicHolidayType.isPublicHoliday)
                    {
                        Console.WriteLine("YES");
                    }
                    else
                    {
                        Console.WriteLine("NO");
                    }
                }

                Console.WriteLine("\r\nAll countries supported by Enrico");
                SupportedCountriesType supportedCountries = client.getSupportedCountries();
                if (!String.IsNullOrEmpty(supportedCountries.error))
                {
                    PrintError(supportedCountries.error);
                }
                else
                {
                    if (supportedCountries.supportedCountries == null || supportedCountries.supportedCountries.Length == 0)
                    {
                        Console.WriteLine("NONE");
                    }
                    else
                    {
                        foreach (SupportedCountryType country in supportedCountries.supportedCountries)
                        {
                            string output = country.fullName + " (" + country.countryCode + "), dates available from " + GetDate(country.fromDate)
                                + ", to " + GetDate(country.toDate);

                            string reg = "";
                            if (country.regions != null && country.regions.Length > 0)
                            {
                                foreach (string region in country.regions)
                                {
                                    if (reg.Length > 0) reg += ", ";
                                    reg += region;
                                }
                            }
                            if (!String.IsNullOrEmpty(reg))
                            {
                                output += ", " + country.regions.Length + " regions (" + reg + ")";
                            }

                            Console.WriteLine(output);
                        }
                    }
                }

                Console.WriteLine("\r\nPress ENTER to exit");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                PrintError(e.Message);
            }
        }

        public static void PrintError(string error)
        {
            Console.WriteLine("ERROR occured: " + error);
        }

        public static void PrintPublicHolidays(PublicHolidayType[] publicHolidays)
        {
            if (publicHolidays != null)
            {
                if (publicHolidays.Length == 0)
                {
                    Console.WriteLine("No holidays");
                }
                else
                {
                    foreach (PublicHolidayType publicHoliday in publicHolidays)
                    {
                        Console.WriteLine(GetDate(publicHoliday.date)
                            + " (" + ConvertDayOfWeek(publicHoliday.date.dayOfWeek) + ")" + " - " + publicHoliday.localName + " (" + publicHoliday.englishName + ")");
                    }
                }
            }
            else
            {
                Console.WriteLine("No holidays");
            }
        }

        public static string ConvertMonth(int month)
        {
            switch (month)
            {
                case 1: return "Jan";
                case 2: return "Feb";
                case 3: return "Mar";
                case 4: return "Apr";
                case 5: return "May";
                case 6: return "Jun";
                case 7: return "Jul";
                case 8: return "Aug";
                case 9: return "Sep";
                case 10: return "Oct";
                case 11: return "Nov";
                case 12: return "Dec";
            }

            return String.Empty;
        }

        public static string ConvertDayOfWeek(int dayOfWeek)
        {
            switch (dayOfWeek)
            {
                case 1: return "Monday";
                case 2: return "Tuesday";
                case 3: return "Wednesday";
                case 4: return "Thursday";
                case 5: return "Friday";
                case 6: return "Saturday";
                case 7: return "Sunday";
            }

            return String.Empty;
        }

        public static string GetDate(DateType date)
        {
            return date.day + " " + ConvertMonth(date.month) + " " + date.year;
        }
    }
}
